import {
  RPCClientCodec,
  RPCClientResponseCodec,
  RPCClientNotificationCodec,
} from '../RPCClientCodec';

import {
  RPCError,
} from '../../error';

export interface ClientNotification {
  method: string;
  params?: any[];
}

export interface ClientRequest {
  jsonrpc: string;
  method: string;
  params?: any[];
  id?: number;
}

export interface ClientResponse {
  jsonrpc: string;
  result?: any;
  error?: RPCError;
  id?: number;
}

export class RPCClientJSONCodec extends RPCClientCodec {
  public request(method: string, args: any[], id?: number): any {
    const req: ClientRequest = {
      jsonrpc: '2.0',
      method: method,
      params: args,
      id: id,
    };
    return JSON.stringify(req);
  }
  public response(res: any): RPCClientResponseCodec {
    const _res: ClientResponse = {
      id: res.id,
      jsonrpc: res.jsonrpc,
      result: res.result,
      error: res.error,
    };
    return new RPCClientJSONResponseCodec(_res);
  }
}

export class RPCClientJSONResponseCodec extends RPCClientResponseCodec {
  public constructor(private _res: ClientResponse) {
    super();
  }

  public id(): number | undefined {
    return this._res.id;
  }
  public error(): RPCError | undefined {
    return this._res.error;
  }
  public result(): any | undefined {
    return this._res.result;
  }

  public isNotification(): boolean {
    if (undefined !== this.id() || undefined === this.result()) {
      return false;
    }
    return true;
  }

  public notification(): RPCClientNotificationCodec | undefined {
    if (undefined !== this.id() || undefined === this.result()) {
      return undefined;
    }
    const _noti: ClientNotification = {
      method: this._res.result.method,
      params: this._res.result.params,
    };
    return new RPCClientJSONNotificationCodec(_noti);
  }
}

export class RPCClientJSONNotificationCodec extends RPCClientNotificationCodec {
  public constructor(private _noti: ClientNotification) {
    super();
  }

  public method(): string {
    return this._noti.method;
  }
  public params(): any[] | undefined {
    return this._noti.params;
  }
}
