import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import * as TotpStore from '../../store/totp';
import { Store, select } from '@ngrx/store';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { TotpSelector } from '../../store';
import {AuthSelector} from "../../../../member/store/index";
import {Member} from "../../../../member/model/Member";

@Component({
  selector: 'of-totp',
  templateUrl: './totp.component.html',
  styleUrls: ['./totp.component.scss']
})
export class TotpComponent implements OnInit {
  pending$ = this.store.pipe(select(TotpSelector.select('pending')));
  error$ = this.store.pipe(select(TotpSelector.select('error')));

  errorMessage: string | null;
  totpForm: FormGroup;
  formErrors = {
    'code': '',
  };
  constructor(private activatedRoute: ActivatedRoute,
              private router: Router,
              private store: Store<TotpStore.State>,
              private formBuilder: FormBuilder,
              ) { }

  ngOnInit() {
    this.totpForm = this.formBuilder.group({
      'code': [
        [
        ]
      ]
    })
    this.pending$.subscribe((pending: boolean) => {
      if (pending) {
        this.totpForm.disable();
      } else {
        this.totpForm.enable();
      }
    });

    this.error$.subscribe((error) => {
      if (error) {
        this.errorMessage = error.exception;
      } else {
        this.errorMessage = null;
      }
    });
  }

  registClick() {
    const code = this.totpForm.value['code'];
    const secretCode = 'X6AWAK573M5372NM';

    this.store.select(AuthSelector.select('member')).subscribe(
      (member: Member) => {
        this.store.dispatch(new TotpStore.Regist({ member, secretCode, code }));
      },
      (error) => {
        console.log(error);
      }
    );
  }
}
